﻿using System;
using Alachisoft.NCache.Runtime;
using Alachisoft.NCache.Runtime.Dependencies;
using Alachisoft.NCache.Web.Caching;

// ===============================================================================
// Alachisoft (R) NCache Sample Code.
// NCache Key Based Dependency Sample.
// ===============================================================================
// Copyright © Alachisoft.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
// ===============================================================================


namespace Dependency
{
	public class KeyBasedDependency
	{

		public static void RunKeyBasedDependencyDemo()
		{
			try
			{
				//Initialize cache
				Cache cache;
				cache = NCache.InitializeCache("mypartitionedcache");
				cache.Clear();

				//Key dependency makes an item to get removed when
				//it's accosiated item is removed or has it's value changed.

				cache.Add("KeyItem1", "This is item 1");

				cache.Add("KeyItem2", "This is item 2", new KeyDependency("KeyItem1"), Cache.NoAbsoluteExpiration, Cache.NoSlidingExpiration, CacheItemPriority.Normal);

				cache.Insert("KeyItem1", "This is item 1 updated");

				object value = cache.Get("KeyItem2");

				if (value == null)
				{
					Console.WriteLine("Dependent item is successfully removed from cache.");
				}
				else
				{
					Console.WriteLine("Error while removing the dependent items from cache.");
				}

				//An item can be dependent on multiple keys!
				//All the keys specified for dependecy must exist in cache

				cache.Add("KeyItem2", "This is item 2");

				cache.Add("KeyItem3", "This is item 3", new KeyDependency(new string[]{"KeyItem1", "KeyItem2"}), Cache.NoAbsoluteExpiration, Cache.NoSlidingExpiration, CacheItemPriority.Normal);

				cache.Insert("KeyItem2", "This is item 2 updated");

				value = cache.Get("KeyItem3");

				if (value == null)
				{
					Console.WriteLine("Dependent item is successfully removed from cache.");
				}
				else
				{
					Console.WriteLine("Error while removing the dependent items from cache.");
				}

				//Key dependency is cascaded
				//If A is dependent of B and B is dependent on C
				//Altering C will remove B and A from the cache.

				//Must dispose cache
				cache.Dispose();


			}
			catch (Exception ex)
			{
				Console.WriteLine(ex.Message);
			}
		}
	}

}